from arkclient import GameBotClient
from arkdriver.lib import Ini
from arkdriver import Admin
from pathlib import Path
from time import sleep

__testing__ = False


def run():
    user_config = Path().cwd() / Path('config.ini')
    default_config = Path(__file__).parent / Path('config.ini')
    config = Ini(user_config) if user_config.exists() else Ini(default_config)
    password = config['ADMIN']['password']
    admin = Admin(password=password)
    admin.enable_admin()
    admin.execute()

    if not __testing__:
        with GameBotClient('bot') as bot:
            while True:
                response = bot.send("ping")
                data = response.decode()
                admin.command_list += data
                admin.execute()
                sleep(5)


if __name__ == "__main__":
    run()



"""
Just a prototype, do not actually use. Its not finished.
"""
import threading
import ctypes
import win32api
import win32gui


class Clipboard:
    def _create_window(self) -> int:
        """
        Create a window for listening to messages
        :return: window hwnd
        """
        wc = win32gui.WNDCLASS()
        wc.lpfnWndProc = self._process_message
        wc.lpszClassName = self.__class__.__name__
        wc.hInstance = win32api.GetModuleHandle(None)
        class_atom = win32gui.RegisterClass(wc)
        return win32gui.CreateWindow(class_atom, self.__class__.__name__, 0, 0, 0, 0, 0, 0, 0, wc.hInstance, None)

    def _process_message(self, hwnd: int, msg: int, wparam: int, lparam: int):
        WM_CLIPBOARDUPDATE = 0x031D
        if msg == WM_CLIPBOARDUPDATE:
            print('clipboard updated!')
        return 0

    def listen(self):
        def runner():
            hwnd = self._create_window()
            ctypes.windll.user32.AddClipboardFormatListener(hwnd)
            win32gui.PumpMessages()

        th = threading.Thread(target=runner, daemon=True)
        th.start()
        while th.is_alive():
            th.join(0.25)


"""
# import ctypes
# import threading
# from dataclasses import dataclass
# from pathlib import Path
# from typing import Callable, Union, List, Optional
#
# import win32api, win32clipboard, win32con, win32gui
#
#
# class Clipboard:
#     @dataclass
#     class Clip:
#         type: str
#         value: Union[str, List[Path]]
#
#     def __init__(
#             self,
#             trigger_at_start: bool = False,
#             on_text: Callable[[str], None] = None,
#             on_update: Callable[[Clip], None] = None,
#             on_files: Callable[[str], None] = None,
#     ):
#         self._trigger_at_start = trigger_at_start
#         self._on_update = on_update
#         self._on_files = on_files
#         self._on_text = on_text
#
#     def _create_window(self) -> int:
#         wc = win32gui.WNDCLASS()
#         wc.lpfnWndProc = self._process_message
#         wc.lpszClassName = self.__class__.__name__
#         wc.hInstance = win32api.GetModuleHandle(None)
#         class_atom = win32gui.RegisterClass(wc)
#         return win32gui.CreateWindow(class_atom, self.__class__.__name__, 0, 0, 0, 0, 0, 0, 0, wc.hInstance, None)
#
#     def _process_message(self, hwnd: int, msg: int, wparam: int, lparam: int):
#         WM_CLIPBOARDUPDATE = 0x031D
#         if msg == WM_CLIPBOARDUPDATE:
#             self._process_clip()
#         return 0
#
#     def _process_clip(self):
#         clip = self.read_clipboard()
#         if not clip:
#             return
#
#         if self._on_update:
#             self._on_update(clip)
#         if clip.type == 'text' and self._on_text:
#             self._on_text(clip.value)
#         elif clip.type == 'files' and self._on_text:
#             self._on_files(clip.value)
#
#     @staticmethod
#     def read_clipboard() -> Optional[Clip]:
#         try:
#             win32clipboard.OpenClipboard()
#
#             def get_formatted(fmt):
#                 if win32clipboard.IsClipboardFormatAvailable(fmt):
#                     return win32clipboard.GetClipboardData(fmt)
#                 return None
#
#             if files := get_formatted(win32con.CF_HDROP):
#                 return Clipboard.Clip('files', [Path(f) for f in files])
#             elif text := get_formatted(win32con.CF_UNICODETEXT):
#                 return Clipboard.Clip('text', text)
#             elif text_bytes := get_formatted(win32con.CF_TEXT):
#                 return Clipboard.Clip('text', text_bytes.decode())
#             elif bitmap_handle := get_formatted(win32con.CF_BITMAP):
#                 # TODO: handle screenshots
#                 pass
#
#             return None
#         finally:
#             win32clipboard.CloseClipboard()
#
#     def listen(self):
#         if self._trigger_at_start:
#             self._process_clip()
#
#         def runner():
#             hwnd = self._create_window()
#             ctypes.windll.user32.AddClipboardFormatListener(hwnd)
#             win32gui.PumpMessages()
#
#         th = threading.Thread(target=runner, daemon=True)
#         th.start()
#         while th.is_alive():
#             th.join(0.25)
#
#
# if __name__ == '__main__':
#     clipboard = Clipboard(on_update=print, trigger_at_start=True)
#     clipboard.listen()
#
#
# if __name__ == '__main__':
#     clipboard = Clipboard()
#     clipboard.listen()
"""



"""
This program controls the admin commands in the game.
Also any menu navigations.


Note: SpecimenImplant has a player's ID called "Sample #: 34225234"
Can have user input their sample #


"""

from time import sleep
# from pywinauto import keyboard, mouse
from arkdriver.driver.application import Application
from pathlib import Path
import json


class Driver:
    def __init__(self, application=None):
        if application is None:
            application = Application()
            application.start()
        self.app = application
        self.pane = application.pane

    def open_players_list(self):
        """ In spectator mode, lists all current players """
        self.app.set_focus()
        sleep(1)
        self.app.send_key_ctrl_n()

    def close_players_list(self):
        """ Closes the players list window """
        coords = self.app.locate_button_by_image('player_list', 'close_template')
        self.app.click(coords=coords)

    def save_players_list(self):
        """ Stores the list of all players """
        # TODO: Machine learning and OCR to get the text list
        pass

    def close_admin_menu(self):
        """ Closes the admin menu window """
        coords = self.app.locate_button_by_image('admin_menu', 'close_template')
        self.app.click(coords=coords)

    def resume_pause_menu(self):
        """ Closes the pause menu window """
        coords = self.app.locate_button_by_image('pause_menu', 'close_template')
        self.app.click(coords=coords)

    def close_pause_menu(self):
        """ Closes the pause menu window """
        coords = self.app.locate_button_by_image('pause_menu', 'resume_template')
        self.app.click(coords=coords)

    def exit_to_main_menu(self):
        """ Exits to the main menu """
        coords = self.app.locate_button_by_image('pause_menu', 'exit_to_main_menu_template')
        self.app.click(coords=coords)

    def target_player_name(self, name):
        """ While in spectator mode, find player and lock onto them """
        # TODO: Machine learning and OCR
        # TODO: Dynamically recognize a list and click to them
        pass

    def scrape_player_data(self):
        """ Gather player blueprint_data (e.g. player_id and tribe_id) """
        # TODO: machine learning to scrape images text and OCR
        pass

    def teleport_default(self):
        """ Teleport to a hidden location """
        # TODO: find a hidden place with x, y, z coodinates
        pass

    def copy_coords(self):
        """ Copies your current coordinates and rotation to clipboard in the form x,y,z Yaw pitch """
        self.write_console("ccc")

    def save_current_coordinates(self, map, name):
        """ Stores the coordinates into file with a name
        :param name: str, name for the coordinates
        """
        data = {"center": {}, "gen1": {}, "gen2": {}, "island": {},
                "scortched": {}, "ragnarok": {}, "extinction": {}, "aberration": {}}
        if map not in data:
            raise NotImplementedError("'{}' isn't part of the map list: {}".format(map, data.keys()))
        self.copy_coords()
        coords = [float(val) for val in self.app.get_from_clipboard().split()]
        file_path = Path.cwd() / Path('../logs/saved_coordinates.json')
        if file_path.exists():
            with open(file_path, 'r') as r:
                data = json.load(r)

        cc = {"x": coords[0], "y": coords[1], "z": coords[2], "yaw": coords[3], "pitch": coords[4]}
        data[map][name] = cc
        with open(file_path, "w") as w:
            json.dump(data, w)

    def write_console(self, *commands):
        self.app.set_focus()
        sleep(1)
        self.app.set_focus()
        self.app.save_to_clipboard_text("|".join(["{}".format(command) for command in commands]) + '|')
        self.app.send_key_tab()
        sleep(0.2)
        self.app.send_key_paste()
        sleep(1)
        self.app.send_key_enter()

    def write_console_args(self, args: list, *command_formats):
        """ player_ids is a list of player ids
            command_formats are commands in format form: "cheat GiveItemTO {} Pickaxe 1 1"
                where there is a '{}' in the string
         """
        commands = []
        for str_format in command_formats:
            commands.append(str_format.format(*args))
        self.write_console(*commands)


def main():
    # TODO: Ensure I'm signed into x
    from application import Application

    app = Application()
    app.start()
    app.wait_exists()
    sleep(5)
    app.set_focus()
    ark_driver = Driver(app)
    ark_driver.click_start()
    sleep(3)
    ark_driver.click_join_ark()
    sleep(10)
    ark_driver.search("mark")
    sleep(2)
    ark_driver.click_refresh()
    sleep(10)
    ark_driver.click_first_search_result()
    ark_driver.click_join()
    input("Press ENTER to exit...")


if __name__ == "__main__":
    main()



from arkdriver.driver.driver import Driver


class Admin:
    def __init__(self, driver=None, password='password', player_id=None):
        # ctrl-x to clear console
        # ctrl-c to copy console
        if driver is None:
            driver = Driver()
        self.__password = password
        self.__my_player_id = player_id
        self.driver = driver
        self.command_list = []
        self.cache = {'default': player_id}

    def execute(self, *commands):
        if len(self.command_list) > 0:
            self.driver.write_console(*self.command_list, *commands)
            self.command_list.clear()

    def enable_admin(self):
        self.enable_cheats(self.__password)
        self.gcm()
        self.ghost()
        return self

    def enable_cheats(self, password):
        self.command_list.append(CreativeMode.ENABLE_CHEATS.format(password))
        return self

    def gcm(self):
        self.command_list.append(CreativeMode.GIVE_CREATIVE_MODE)
        return self

    def ghost(self):
        """ Allows to go through walls """
        self.command_list.append(PlayerManagement.GHOST)
        self.command_list.append(PlayerManagement.LEAVE_ME_ALONE)
        self.command_list.append(PlayerManagement.GOD)
        self.command_list.append(PlayerManagement.CLEAR_MY_BUFFS)
        self.command_list.append(PlayerManagement.REFILL_STATS)
        return self

    def enable_spectator(self):
        """ Once you enter the spectator state your body will disapear and your inventory items
        will be erased as well. If you leave the game and rejoin it will carry on where you left
        off, thus using the command "stopspectating" is highly recommended. The latter will
        show the respawn map. """
        self.command_list.append(Spectator.ENABLE_SPECTATOR)
        return self

    def stop_spectating(self):
        """ Stops spectating """
        self.command_list.append(Spectator.STOP_SPECTATING)
        return self

    def message_server(self, message):
        """ Writes to chat as a server message to all players"""
        self.command_list.append(ServerManagement.SERVER_CHAT.format(message))
        return self

    def broadcast(self, message):
        """ Sends a message to the whole server using the banner """
        self.command_list.append(ServerManagement.BROADCAST.format(str(message)))
        return self

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.execute()

    def restart_application(self):
        self.driver.app.close()
        return Admin(password=self.__password)


class CreativeMode:
    ENABLE_CHEATS = "EnableCheats {}"
    GIVE_CREATIVE_MODE = "cheat GCM"


class Spectator:
    ENABLE_SPECTATOR = "cheat EnableSpectator"
    STOP_SPECTATING = "cheat StopSpectating"


class ServerManagement:
    BROADCAST = "cheat Broadcast {}"
    SERVER_CHAT = "cheat ServerChat {}"
    SERVER_CHAT_TO = "cheat ServerChatTo {} {}"
    SERVER_CHAT_TO_Player = "cheat ServerChatToPlayer {} {}"


class Teleportation:
    TELEPORT = "cheat Teleport"
    TELEPORT_PLAYER_ID_TO_ME = "cheat TeleportPlayerIdToMe {}"
    TELEPORT_PLAYER_NAME_TO_ME = "cheat TeleportPlayerNameToMe {}"
    TELEPORT_TO_PLAYER = "cheat TeleportToPlayer {}"
    TP = "cheat TP {}"
    TP_COORDS = "cheat TPCoords {} {} {}"
    SET_PLAYER_POS = "cheat SetPlayerPos {} {} {}"


class PlayerManagement:
    FLY = "cheat Fly"
    GHOST = "cheat Ghost"
    GOD = "cheat God"
    INFINITE_STATS = "cheat InfiniteStats"
    LEAVE_ME_ALONE = "cheat LeaveMeAlone"
    REFILL_STATS = "cheat RefillStats"
    CLEAR_MY_BUFFS = "cheat ClearMyBuffs"



from arkdriver.driver import Admin, Application, Driver
from arkdriver.server import GameBotServer
from arkdriver.main import run
from arkdriver.lib import console

__all__ = ['Admin', "run", 'console', 'GameBotServer', 'Application', 'Driver']
